# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/ydb_formats.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18protos/ydb_formats.proto\x12\x0bYdb.Formats\"$\n\x12\x41rrowBatchSettings\x12\x0e\n\x06schema\x18\x01 \x01(\x0c\"W\n\x0b\x43svSettings\x12\x11\n\tskip_rows\x18\x01 \x01(\r\x12\x11\n\tdelimiter\x18\x02 \x01(\x0c\x12\x12\n\nnull_value\x18\x03 \x01(\x0c\x12\x0e\n\x06header\x18\x04 \x01(\x08\x42Q\n\x10tech.ydb.formatsZ:github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Formats\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protos.ydb_formats_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\020tech.ydb.formatsZ:github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Formats\370\001\001'
  _ARROWBATCHSETTINGS._serialized_start=41
  _ARROWBATCHSETTINGS._serialized_end=77
  _CSVSETTINGS._serialized_start=79
  _CSVSETTINGS._serialized_end=166
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/ydb_query_stats.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protos/ydb_query_stats.proto',
  package='Ydb.TableStats',
  syntax='proto3',
  serialized_options=b'\n\010tech.ydbZ=github.com/ydb-platform/ydb-go-genproto/protos/Ydb_TableStats\370\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1cprotos/ydb_query_stats.proto\x12\x0eYdb.TableStats\"-\n\x0eOperationStats\x12\x0c\n\x04rows\x18\x01 \x01(\x04\x12\r\n\x05\x62ytes\x18\x02 \x01(\x04\"\xd1\x01\n\x10TableAccessStats\x12\x0c\n\x04name\x18\x01 \x01(\t\x12-\n\x05reads\x18\x03 \x01(\x0b\x32\x1e.Ydb.TableStats.OperationStats\x12/\n\x07updates\x18\x04 \x01(\x0b\x32\x1e.Ydb.TableStats.OperationStats\x12/\n\x07\x64\x65letes\x18\x05 \x01(\x0b\x32\x1e.Ydb.TableStats.OperationStats\x12\x18\n\x10partitions_count\x18\x06 \x01(\x04J\x04\x08\x02\x10\x03\"\xa3\x01\n\x0fQueryPhaseStats\x12\x13\n\x0b\x64uration_us\x18\x01 \x01(\x04\x12\x36\n\x0ctable_access\x18\x02 \x03(\x0b\x32 .Ydb.TableStats.TableAccessStats\x12\x13\n\x0b\x63pu_time_us\x18\x03 \x01(\x04\x12\x17\n\x0f\x61\x66\x66\x65\x63ted_shards\x18\x04 \x01(\x04\x12\x15\n\rliteral_phase\x18\x05 \x01(\x08\"P\n\x10\x43ompilationStats\x12\x12\n\nfrom_cache\x18\x01 \x01(\x08\x12\x13\n\x0b\x64uration_us\x18\x02 \x01(\x04\x12\x13\n\x0b\x63pu_time_us\x18\x03 \x01(\x04\"\xf4\x01\n\nQueryStats\x12\x35\n\x0cquery_phases\x18\x01 \x03(\x0b\x32\x1f.Ydb.TableStats.QueryPhaseStats\x12\x35\n\x0b\x63ompilation\x18\x02 \x01(\x0b\x32 .Ydb.TableStats.CompilationStats\x12\x1b\n\x13process_cpu_time_us\x18\x03 \x01(\x04\x12\x12\n\nquery_plan\x18\x04 \x01(\t\x12\x11\n\tquery_ast\x18\x05 \x01(\t\x12\x19\n\x11total_duration_us\x18\x06 \x01(\x04\x12\x19\n\x11total_cpu_time_us\x18\x07 \x01(\x04\x42L\n\x08tech.ydbZ=github.com/ydb-platform/ydb-go-genproto/protos/Ydb_TableStats\xf8\x01\x01\x62\x06proto3'
)




_OPERATIONSTATS = _descriptor.Descriptor(
  name='OperationStats',
  full_name='Ydb.TableStats.OperationStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='rows', full_name='Ydb.TableStats.OperationStats.rows', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bytes', full_name='Ydb.TableStats.OperationStats.bytes', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=48,
  serialized_end=93,
)


_TABLEACCESSSTATS = _descriptor.Descriptor(
  name='TableAccessStats',
  full_name='Ydb.TableStats.TableAccessStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Ydb.TableStats.TableAccessStats.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='reads', full_name='Ydb.TableStats.TableAccessStats.reads', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='updates', full_name='Ydb.TableStats.TableAccessStats.updates', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deletes', full_name='Ydb.TableStats.TableAccessStats.deletes', index=3,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='partitions_count', full_name='Ydb.TableStats.TableAccessStats.partitions_count', index=4,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=96,
  serialized_end=305,
)


_QUERYPHASESTATS = _descriptor.Descriptor(
  name='QueryPhaseStats',
  full_name='Ydb.TableStats.QueryPhaseStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='duration_us', full_name='Ydb.TableStats.QueryPhaseStats.duration_us', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table_access', full_name='Ydb.TableStats.QueryPhaseStats.table_access', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cpu_time_us', full_name='Ydb.TableStats.QueryPhaseStats.cpu_time_us', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='affected_shards', full_name='Ydb.TableStats.QueryPhaseStats.affected_shards', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='literal_phase', full_name='Ydb.TableStats.QueryPhaseStats.literal_phase', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=308,
  serialized_end=471,
)


_COMPILATIONSTATS = _descriptor.Descriptor(
  name='CompilationStats',
  full_name='Ydb.TableStats.CompilationStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='from_cache', full_name='Ydb.TableStats.CompilationStats.from_cache', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='duration_us', full_name='Ydb.TableStats.CompilationStats.duration_us', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cpu_time_us', full_name='Ydb.TableStats.CompilationStats.cpu_time_us', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=473,
  serialized_end=553,
)


_QUERYSTATS = _descriptor.Descriptor(
  name='QueryStats',
  full_name='Ydb.TableStats.QueryStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='query_phases', full_name='Ydb.TableStats.QueryStats.query_phases', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='compilation', full_name='Ydb.TableStats.QueryStats.compilation', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='process_cpu_time_us', full_name='Ydb.TableStats.QueryStats.process_cpu_time_us', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='query_plan', full_name='Ydb.TableStats.QueryStats.query_plan', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='query_ast', full_name='Ydb.TableStats.QueryStats.query_ast', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='total_duration_us', full_name='Ydb.TableStats.QueryStats.total_duration_us', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='total_cpu_time_us', full_name='Ydb.TableStats.QueryStats.total_cpu_time_us', index=6,
      number=7, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=556,
  serialized_end=800,
)

_TABLEACCESSSTATS.fields_by_name['reads'].message_type = _OPERATIONSTATS
_TABLEACCESSSTATS.fields_by_name['updates'].message_type = _OPERATIONSTATS
_TABLEACCESSSTATS.fields_by_name['deletes'].message_type = _OPERATIONSTATS
_QUERYPHASESTATS.fields_by_name['table_access'].message_type = _TABLEACCESSSTATS
_QUERYSTATS.fields_by_name['query_phases'].message_type = _QUERYPHASESTATS
_QUERYSTATS.fields_by_name['compilation'].message_type = _COMPILATIONSTATS
DESCRIPTOR.message_types_by_name['OperationStats'] = _OPERATIONSTATS
DESCRIPTOR.message_types_by_name['TableAccessStats'] = _TABLEACCESSSTATS
DESCRIPTOR.message_types_by_name['QueryPhaseStats'] = _QUERYPHASESTATS
DESCRIPTOR.message_types_by_name['CompilationStats'] = _COMPILATIONSTATS
DESCRIPTOR.message_types_by_name['QueryStats'] = _QUERYSTATS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

OperationStats = _reflection.GeneratedProtocolMessageType('OperationStats', (_message.Message,), {
  'DESCRIPTOR' : _OPERATIONSTATS,
  '__module__' : 'protos.ydb_query_stats_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.TableStats.OperationStats)
  })
_sym_db.RegisterMessage(OperationStats)

TableAccessStats = _reflection.GeneratedProtocolMessageType('TableAccessStats', (_message.Message,), {
  'DESCRIPTOR' : _TABLEACCESSSTATS,
  '__module__' : 'protos.ydb_query_stats_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.TableStats.TableAccessStats)
  })
_sym_db.RegisterMessage(TableAccessStats)

QueryPhaseStats = _reflection.GeneratedProtocolMessageType('QueryPhaseStats', (_message.Message,), {
  'DESCRIPTOR' : _QUERYPHASESTATS,
  '__module__' : 'protos.ydb_query_stats_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.TableStats.QueryPhaseStats)
  })
_sym_db.RegisterMessage(QueryPhaseStats)

CompilationStats = _reflection.GeneratedProtocolMessageType('CompilationStats', (_message.Message,), {
  'DESCRIPTOR' : _COMPILATIONSTATS,
  '__module__' : 'protos.ydb_query_stats_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.TableStats.CompilationStats)
  })
_sym_db.RegisterMessage(CompilationStats)

QueryStats = _reflection.GeneratedProtocolMessageType('QueryStats', (_message.Message,), {
  'DESCRIPTOR' : _QUERYSTATS,
  '__module__' : 'protos.ydb_query_stats_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.TableStats.QueryStats)
  })
_sym_db.RegisterMessage(QueryStats)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
import logging
import copy
from concurrent import futures
import uuid
import threading
import collections

from google.protobuf import text_format
import grpc
from . import issues, _apis, _utilities
from . import default_pem

_stubs_list = (
    _apis.TableService.Stub,
    _apis.SchemeService.Stub,
    _apis.DiscoveryService.Stub,
    _apis.CmsService.Stub,
)

logger = logging.getLogger(__name__)
DEFAULT_TIMEOUT = 600
YDB_DATABASE_HEADER = "x-ydb-database"
YDB_TRACE_ID_HEADER = "x-ydb-trace-id"
YDB_REQUEST_TYPE_HEADER = "x-ydb-request-type"


def _message_to_string(message):
    """
    Constructs a string representation of provided message or generator
    :param message: A protocol buffer or generator instance
    :return: A string
    """
    try:
        return text_format.MessageToString(message, as_one_line=True)
    except Exception:
        return str(message)


def _log_response(rpc_state, response):
    """
    Writes a message with response into debug logs
    :param rpc_state: A state of rpc
    :param response: A received response
    :return: None
    """
    if logger.isEnabledFor(logging.DEBUG):
        logger.debug("%s: response = { %s }", rpc_state, _message_to_string(response))


def _log_request(rpc_state, request):
    """
    Writes a message with request into debug logs
    :param rpc_state: An id of request
    :param request: A received response
    :return: None
    """
    if logger.isEnabledFor(logging.DEBUG):
        logger.debug("%s: request = { %s }", rpc_state, _message_to_string(request))


def _rpc_error_handler(rpc_state, rpc_error, on_disconnected=None):
    """
    RPC call error handler, that translates gRPC error into YDB issue
    :param rpc_state: A state of rpc
    :param rpc_error: an underlying rpc error to handle
    :param on_disconnected: a handler to call on disconnected connection
    """
    logger.info("%s: received error, %s", rpc_state, rpc_error)
    if isinstance(rpc_error, grpc.Call):
        if rpc_error.code() == grpc.StatusCode.UNAUTHENTICATED:
            return issues.Unauthenticated(rpc_error.details())
        elif rpc_error.code() == grpc.StatusCode.DEADLINE_EXCEEDED:
            return issues.DeadlineExceed("Deadline exceeded on request")
        elif rpc_error.code() == grpc.StatusCode.UNIMPLEMENTED:
            return issues.Unimplemented(
                "Method or feature is not implemented on server!"
            )

    logger.debug("%s: unhandled rpc error, disconnecting channel", rpc_state)
    if on_disconnected is not None:
        on_disconnected()

    return issues.ConnectionLost("Rpc error, reason %s" % str(rpc_error))


def _on_response_callback(
    rpc_state, call_state_unref, wrap_result=None, on_disconnected=None, wrap_args=()
):
    """
    Callback to be executed on received RPC response
    :param rpc_state: A name of RPC
    :param wrap_result: A callable that wraps received response
    :param on_disconnected: A handler to executed on disconnected channel
    :param wrap_args: An arguments to be passed into wrap result callable
    :return: None
    """
    try:
        logger.debug("%s: on response callback started", rpc_state)
        response = rpc_state.rendezvous.result()
        _log_response(rpc_state, response)
        response = (
            response
            if wrap_result is None
            else wrap_result(rpc_state, response, *wrap_args)
        )
        rpc_state.result_future.set_result(response)
        logger.debug("%s: on response callback success", rpc_state)
    except grpc.FutureCancelledError as e:
        logger.debug("%s: request execution cancelled", rpc_state)
        if not rpc_state.result_future.cancelled():
            rpc_state.result_future.set_exception(e)

    except grpc.RpcError as rpc_call_error:
        rpc_state.result_future.set_exception(
            _rpc_error_handler(rpc_state, rpc_call_error, on_disconnected)
        )

    except issues.Error as e:
        logger.info("%s: received exception, %s", rpc_state, str(e))
        rpc_state.result_future.set_exception(e)

    except Exception as e:
        logger.error("%s: received exception, %s", rpc_state, str(e))
        rpc_state.result_future.set_exception(issues.ConnectionLost(str(e)))

    call_state_unref()


def _construct_metadata(driver_config, settings):
    """
    Translates request settings into RPC metadata
    :param driver_config: A driver config
    :param settings: An instance of BaseRequestSettings
    :return: RPC metadata
    """
    metadata = []
    if driver_config.database is not None:
        metadata.append((YDB_DATABASE_HEADER, driver_config.database))

    need_rpc_auth = getattr(settings, "need_rpc_auth", True)
    if driver_config.credentials is not None and need_rpc_auth:
        metadata.extend(driver_config.credentials.auth_metadata())

    if settings is not None:
        if settings.trace_id is not None:
            metadata.append((YDB_TRACE_ID_HEADER, settings.trace_id))
        if settings.request_type is not None:
            metadata.append((YDB_REQUEST_TYPE_HEADER, settings.request_type))
        metadata.extend(getattr(settings, "headers", []))

    metadata.append(_utilities.x_ydb_sdk_build_info_header())
    return metadata


def _get_request_timeout(settings):
    """
    Extracts RPC timeout from request settings
    :param settings: an instance of BaseRequestSettings
    :return: timeout of RPC execution
    """
    if settings is None or settings.timeout is None:
        return DEFAULT_TIMEOUT
    return settings.timeout


class EndpointOptions(object):
    __slots__ = ("ssl_target_name_override", "node_id")

    def __init__(self, ssl_target_name_override=None, node_id=None):
        self.ssl_target_name_override = ssl_target_name_override
        self.node_id = node_id


def _construct_channel_options(driver_config, endpoint_options=None):
    """
    Constructs gRPC channel initialization options
    :param driver_config: A driver config instance
    :param endpoint_options: Endpoint options
    :return: A channel initialization options
    """
    _max_message_size = 64 * 10**6
    _default_connect_options = [
        ("grpc.max_receive_message_length", _max_message_size),
        ("grpc.max_send_message_length", _max_message_size),
        ("grpc.primary_user_agent", driver_config.primary_user_agent),
        (
            "grpc.lb_policy_name",
            getattr(driver_config, "grpc_lb_policy_name", "round_robin"),
        ),
    ]
    if driver_config.grpc_keep_alive_timeout is not None:
        _default_connect_options.extend(
            [
                ("grpc.keepalive_time_ms", driver_config.grpc_keep_alive_timeout >> 3),
                ("grpc.keepalive_timeout_ms", driver_config.grpc_keep_alive_timeout),
                ("grpc.http2.max_pings_without_data", 0),
                ("grpc.keepalive_permit_without_calls", 0),
            ]
        )
    if endpoint_options is not None:
        if endpoint_options.ssl_target_name_override:
            _default_connect_options.append(
                (
                    "grpc.ssl_target_name_override",
                    endpoint_options.ssl_target_name_override,
                )
            )
    if driver_config.channel_options is None:
        return _default_connect_options
    channel_options = copy.deepcopy(driver_config.channel_options)
    custom_options_keys = set(i[0] for i in driver_config.channel_options)
    for item in filter(
        lambda x: x[0] not in custom_options_keys, _default_connect_options
    ):
        channel_options.append(item)
    return channel_options


class _RpcState(object):
    __slots__ = (
        "rpc",
        "request_id",
        "result_future",
        "rpc_name",
        "endpoint",
        "rendezvous",
        "metadata_kv",
        "endpoint_key",
    )

    def __init__(self, stub_instance, rpc_name, endpoint, endpoint_key):
        """Stores all RPC related data"""
        self.rpc_name = rpc_name
        self.rpc = getattr(stub_instance, rpc_name)
        self.request_id = uuid.uuid4()
        self.endpoint = endpoint
        self.rendezvous = None
        self.metadata_kv = None
        self.endpoint_key = endpoint_key

    def __str__(self):
        return "RpcState(%s, %s, %s)" % (self.rpc_name, self.request_id, self.endpoint)

    def __call__(self, *args, **kwargs):
        """Execute a RPC."""
        try:
            response, rendezvous = self.rpc.with_call(*args, **kwargs)
            self.rendezvous = rendezvous
            return response
        except AttributeError:
            return self.rpc(*args, **kwargs)

    def trailing_metadata(self):
        """Trailing metadata of the call."""
        if self.metadata_kv is None:

            self.metadata_kv = collections.defaultdict(set)
            for metadatum in self.rendezvous.trailing_metadata():
                self.metadata_kv[metadatum.key].add(metadatum.value)

        return self.metadata_kv

    def future(self, *args, **kwargs):
        self.rendezvous = self.rpc.future(*args, **kwargs)
        self.result_future = futures.Future()

        def _cancel_callback(f):
            """forwards cancel to gPRC future"""
            if f.cancelled():
                self.rendezvous.cancel()

        self.rendezvous.add_done_callback(_cancel_callback)
        return self.rendezvous, self.result_future


_nanos_in_second = 10**9


def _set_duration(duration_value, seconds_float):
    duration_value.seconds = int(seconds_float)
    duration_value.nanos = int((seconds_float - int(seconds_float)) * _nanos_in_second)
    return duration_value


def _set_server_timeouts(request, settings, default_value):
    if not hasattr(request, "operation_params"):
        return

    operation_timeout = getattr(settings, "operation_timeout", default_value)
    operation_timeout = (
        default_value if operation_timeout is None else operation_timeout
    )
    cancel_after = getattr(settings, "cancel_after", default_value)
    cancel_after = default_value if cancel_after is None else cancel_after
    _set_duration(request.operation_params.operation_timeout, operation_timeout)
    _set_duration(request.operation_params.cancel_after, cancel_after)


def channel_factory(
    endpoint, driver_config, channel_provider=None, endpoint_options=None
):
    channel_provider = channel_provider if channel_provider is not None else grpc
    options = _construct_channel_options(driver_config, endpoint_options)
    logger.debug("Channel options: {}".format(options))

    if driver_config.root_certificates is None and not driver_config.secure_channel:
        return channel_provider.insecure_channel(
            endpoint, options, compression=getattr(driver_config, "compression", None)
        )

    root_certificates = driver_config.root_certificates
    if root_certificates is None:
        root_certificates = default_pem.load_default_pem()
    credentials = grpc.ssl_channel_credentials(
        root_certificates, driver_config.private_key, driver_config.certificate_chain
    )
    return channel_provider.secure_channel(
        endpoint,
        credentials,
        options,
        compression=getattr(driver_config, "compression", None),
    )


class EndpointKey(object):
    __slots__ = ("endpoint", "node_id")

    def __init__(self, endpoint, node_id):
        self.endpoint = endpoint
        self.node_id = node_id


class Connection(object):
    __slots__ = (
        "endpoint",
        "_channel",
        "_call_states",
        "_stub_instances",
        "_driver_config",
        "_cleanup_callbacks",
        "__weakref__",
        "lock",
        "calls",
        "closing",
        "endpoint_key",
        "node_id",
    )

    def __init__(self, endpoint, driver_config=None, endpoint_options=None):
        """
        Object that wraps gRPC channel and encapsulates gRPC request execution logic
        :param endpoint: endpoint to connect (in pattern host:port), constructed by user or
        discovered by the YDB endpoint discovery mechanism
        :param driver_config: A driver config instance to be used for RPC call interception
        """
        global _stubs_list
        self.endpoint = endpoint
        self.node_id = getattr(endpoint_options, "node_id", None)
        self.endpoint_key = EndpointKey(
            endpoint, getattr(endpoint_options, "node_id", None)
        )
        self._channel = channel_factory(
            self.endpoint, driver_config, endpoint_options=endpoint_options
        )
        self._driver_config = driver_config
        self._call_states = {}
        self._stub_instances = {}
        self._cleanup_callbacks = []
        # pre-initialize stubs
        for stub in _stubs_list:
            self._stub_instances[stub] = stub(self._channel)
        self.lock = threading.RLock()
        self.calls = 0
        self.closing = False

    def _prepare_stub_instance(self, stub):
        if stub not in self._stub_instances:
            self._stub_instances[stub] = stub(self._channel)

    def add_cleanup_callback(self, callback):
        self._cleanup_callbacks.append(callback)

    def _prepare_call(self, stub, rpc_name, request, settings):
        timeout, metadata = _get_request_timeout(settings), _construct_metadata(
            self._driver_config, settings
        )
        _set_server_timeouts(request, settings, timeout)
        self._prepare_stub_instance(stub)
        rpc_state = _RpcState(
            self._stub_instances[stub], rpc_name, self.endpoint, self.endpoint_key
        )
        logger.debug("%s: creating call state", rpc_state)
        with self.lock:
            if self.closing:
                raise issues.ConnectionLost("Couldn't start call")
            self.calls += 1
            self._call_states[rpc_state.request_id] = rpc_state
        # Call successfully prepared and registered
        _log_request(rpc_state, request)
        return rpc_state, timeout, metadata

    def _finish_call(self, call_state):
        with self.lock:
            self.calls -= 1
            self._call_states.pop(call_state.request_id, None)
            # Call successfully finished
            if self.closing and self.calls == 0:
                # Channel is closing and we have to destroy channel
                self.destroy()

    def future(
        self,
        request,
        stub,
        rpc_name,
        wrap_result=None,
        settings=None,
        wrap_args=(),
        on_disconnected=None,
    ):
        """
        Sends request constructed by client
        :param request: A request constructed by client
        :param stub: A stub instance to wrap channel
        :param rpc_name: A name of RPC to be executed
        :param wrap_result: A callable that intercepts call and wraps received response
        :param settings: An instance of BaseRequestSettings that can be used
        for RPC metadata construction
        :param on_disconnected: A callable to be executed when underlying channel becomes disconnected
        :param wrap_args: And arguments to be passed into wrap_result callable
        :return: A future of computation
        """
        rpc_state, timeout, metadata = self._prepare_call(
            stub, rpc_name, request, settings
        )
        rendezvous, result_future = rpc_state.future(
            request,
            timeout,
            metadata,
            compression=getattr(settings, "compression", None),
        )
        rendezvous.add_done_callback(
            lambda resp_future: _on_response_callback(
                rpc_state,
                lambda: self._finish_call(rpc_state),
                wrap_result,
                on_disconnected,
                wrap_args,
            )
        )
        return result_future

    def __call__(
        self,
        request,
        stub,
        rpc_name,
        wrap_result=None,
        settings=None,
        wrap_args=(),
        on_disconnected=None,
    ):
        """
        Synchronously sends request constructed by client library
        :param request: A request constructed by client
        :param stub: A stub instance to wrap channel
        :param rpc_name: A name of RPC to be executed
        :param wrap_result: A callable that intercepts call and wraps received response
        :param settings: An instance of BaseRequestSettings that can be used
        for RPC metadata construction
        :param on_disconnected: A callable to be executed when underlying channel becomes disconnected
        :param wrap_args: And arguments to be passed into wrap_result callable
        :return: A result of computation
        """
        rpc_state, timeout, metadata = self._prepare_call(
            stub, rpc_name, request, settings
        )
        try:
            response = rpc_state(
                request,
                timeout,
                metadata,
                compression=getattr(settings, "compression", None),
            )
            _log_response(rpc_state, response)
            return (
                response
                if wrap_result is None
                else wrap_result(rpc_state, response, *wrap_args)
            )
        except grpc.RpcError as rpc_error:
            raise _rpc_error_handler(rpc_state, rpc_error, on_disconnected)
        finally:
            self._finish_call(rpc_state)

    @classmethod
    def ready_factory(
        cls, endpoint, driver_config, ready_timeout=10, endpoint_options=None
    ):
        candidate = cls(endpoint, driver_config, endpoint_options=endpoint_options)
        ready_future = candidate.ready_future()
        try:
            ready_future.result(timeout=ready_timeout)
            return candidate
        except grpc.FutureTimeoutError:
            ready_future.cancel()
            candidate.close()
            return None

        except Exception:
            candidate.close()
            return None

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def close(self):
        """
        Closes the underlying gRPC channel
        :return: None
        """
        logger.info("Closing channel for endpoint %s", self.endpoint)
        with self.lock:
            self.closing = True

            for callback in self._cleanup_callbacks:
                callback(self)

            # potentially we should cancel in-flight calls here but currently
            # it is not required since gRPC can successfully cancel these calls manually.

            if self.calls == 0:
                # everything is cancelled/completed and channel can be destroyed
                self.destroy()

    def destroy(self):
        if hasattr(self, "_channel") and hasattr(self._channel, "close"):
            self._channel.close()

    def ready_future(self):
        """
        Creates a future that tracks underlying gRPC channel is ready
        :return: A Future object that matures when the underlying channel is ready
        to receive request
        """
        return grpc.channel_ready_future(self._channel)



# -*- coding: utf-8 -*-
from google.protobuf import text_format
import enum
from six.moves import queue

from . import _apis


_TRANSPORT_STATUSES_FIRST = 401000
_CLIENT_STATUSES_FIRST = 402000


@enum.unique
class StatusCode(enum.IntEnum):
    STATUS_CODE_UNSPECIFIED = _apis.StatusIds.STATUS_CODE_UNSPECIFIED
    SUCCESS = _apis.StatusIds.SUCCESS
    BAD_REQUEST = _apis.StatusIds.BAD_REQUEST
    UNAUTHORIZED = _apis.StatusIds.UNAUTHORIZED
    INTERNAL_ERROR = _apis.StatusIds.INTERNAL_ERROR
    ABORTED = _apis.StatusIds.ABORTED
    UNAVAILABLE = _apis.StatusIds.UNAVAILABLE
    OVERLOADED = _apis.StatusIds.OVERLOADED
    SCHEME_ERROR = _apis.StatusIds.SCHEME_ERROR
    GENERIC_ERROR = _apis.StatusIds.GENERIC_ERROR
    TIMEOUT = _apis.StatusIds.TIMEOUT
    BAD_SESSION = _apis.StatusIds.BAD_SESSION
    PRECONDITION_FAILED = _apis.StatusIds.PRECONDITION_FAILED
    ALREADY_EXISTS = _apis.StatusIds.ALREADY_EXISTS
    NOT_FOUND = _apis.StatusIds.NOT_FOUND
    SESSION_EXPIRED = _apis.StatusIds.SESSION_EXPIRED
    CANCELLED = _apis.StatusIds.CANCELLED
    UNDETERMINED = _apis.StatusIds.UNDETERMINED
    UNSUPPORTED = _apis.StatusIds.UNSUPPORTED
    SESSION_BUSY = _apis.StatusIds.SESSION_BUSY

    CONNECTION_LOST = _TRANSPORT_STATUSES_FIRST + 10
    CONNECTION_FAILURE = _TRANSPORT_STATUSES_FIRST + 20
    DEADLINE_EXCEEDED = _TRANSPORT_STATUSES_FIRST + 30
    CLIENT_INTERNAL_ERROR = _TRANSPORT_STATUSES_FIRST + 40
    UNIMPLEMENTED = _TRANSPORT_STATUSES_FIRST + 50

    UNAUTHENTICATED = _CLIENT_STATUSES_FIRST + 30
    SESSION_POOL_EMPTY = _CLIENT_STATUSES_FIRST + 40


class Error(Exception):
    status = None

    def __init__(self, message, issues=None):
        super(Error, self).__init__(message)
        self.issues = issues
        self.message = message


class TruncatedResponseError(Error):
    status = None


class ConnectionError(Error):
    status = None


class ConnectionFailure(ConnectionError):
    status = StatusCode.CONNECTION_FAILURE


class ConnectionLost(ConnectionError):
    status = StatusCode.CONNECTION_LOST


class DeadlineExceed(ConnectionError):
    status = StatusCode.DEADLINE_EXCEEDED


class Unimplemented(ConnectionError):
    status = StatusCode.UNIMPLEMENTED


class Unauthenticated(Error):
    status = StatusCode.UNAUTHENTICATED


class BadRequest(Error):
    status = StatusCode.BAD_REQUEST


class Unauthorized(Error):
    status = StatusCode.UNAUTHORIZED


class InternalError(Error):
    status = StatusCode.INTERNAL_ERROR


class Aborted(Error):
    status = StatusCode.ABORTED


class Unavailable(Error):
    status = StatusCode.UNAVAILABLE


class Overloaded(Error):
    status = StatusCode.OVERLOADED


class SchemeError(Error):
    status = StatusCode.SCHEME_ERROR


class GenericError(Error):
    status = StatusCode.GENERIC_ERROR


class BadSession(Error):
    status = StatusCode.BAD_SESSION


class Timeout(Error):
    status = StatusCode.TIMEOUT


class PreconditionFailed(Error):
    status = StatusCode.PRECONDITION_FAILED


class NotFound(Error):
    status = StatusCode.NOT_FOUND


class AlreadyExists(Error):
    status = StatusCode.ALREADY_EXISTS


class SessionExpired(Error):
    status = StatusCode.SESSION_EXPIRED


class Cancelled(Error):
    status = StatusCode.CANCELLED


class Undetermined(Error):
    status = StatusCode.UNDETERMINED


class Unsupported(Error):
    status = StatusCode.UNSUPPORTED


class SessionBusy(Error):
    status = StatusCode.SESSION_BUSY


class SessionPoolEmpty(Error, queue.Empty):
    status = StatusCode.SESSION_POOL_EMPTY


def _format_issues(issues):
    if not issues:
        return ""
    return " ,".join(
        [text_format.MessageToString(issue, False, True) for issue in issues]
    )


_success_status_codes = {StatusCode.STATUS_CODE_UNSPECIFIED, StatusCode.SUCCESS}
_server_side_error_map = {
    StatusCode.BAD_REQUEST: BadRequest,
    StatusCode.UNAUTHORIZED: Unauthorized,
    StatusCode.INTERNAL_ERROR: InternalError,
    StatusCode.ABORTED: Aborted,
    StatusCode.UNAVAILABLE: Unavailable,
    StatusCode.OVERLOADED: Overloaded,
    StatusCode.SCHEME_ERROR: SchemeError,
    StatusCode.GENERIC_ERROR: GenericError,
    StatusCode.TIMEOUT: Timeout,
    StatusCode.BAD_SESSION: BadSession,
    StatusCode.PRECONDITION_FAILED: PreconditionFailed,
    StatusCode.ALREADY_EXISTS: AlreadyExists,
    StatusCode.NOT_FOUND: NotFound,
    StatusCode.SESSION_EXPIRED: SessionExpired,
    StatusCode.CANCELLED: Cancelled,
    StatusCode.UNDETERMINED: Undetermined,
    StatusCode.UNSUPPORTED: Unsupported,
    StatusCode.SESSION_BUSY: SessionBusy,
}


def _process_response(response_proto):
    if response_proto.status not in _success_status_codes:
        exc_obj = _server_side_error_map.get(response_proto.status)
        raise exc_obj(_format_issues(response_proto.issues), response_proto.issues)



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/ydb_status_codes.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protos/ydb_status_codes.proto',
  package='Ydb',
  syntax='proto3',
  serialized_options=b'\n\010tech.ydbB\021StatusCodesProtosZ2github.com/ydb-platform/ydb-go-genproto/protos/Ydb',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1dprotos/ydb_status_codes.proto\x12\x03Ydb\"\xa7\x03\n\tStatusIds\"\x99\x03\n\nStatusCode\x12\x1b\n\x17STATUS_CODE_UNSPECIFIED\x10\x00\x12\r\n\x07SUCCESS\x10\x80\xb5\x18\x12\x11\n\x0b\x42\x41\x44_REQUEST\x10\x8a\xb5\x18\x12\x12\n\x0cUNAUTHORIZED\x10\x94\xb5\x18\x12\x14\n\x0eINTERNAL_ERROR\x10\x9e\xb5\x18\x12\r\n\x07\x41\x42ORTED\x10\xa8\xb5\x18\x12\x11\n\x0bUNAVAILABLE\x10\xb2\xb5\x18\x12\x10\n\nOVERLOADED\x10\xbc\xb5\x18\x12\x12\n\x0cSCHEME_ERROR\x10\xc6\xb5\x18\x12\x13\n\rGENERIC_ERROR\x10\xd0\xb5\x18\x12\r\n\x07TIMEOUT\x10\xda\xb5\x18\x12\x11\n\x0b\x42\x41\x44_SESSION\x10\xe4\xb5\x18\x12\x19\n\x13PRECONDITION_FAILED\x10\xf8\xb5\x18\x12\x14\n\x0e\x41LREADY_EXISTS\x10\x82\xb6\x18\x12\x0f\n\tNOT_FOUND\x10\x8c\xb6\x18\x12\x15\n\x0fSESSION_EXPIRED\x10\x96\xb6\x18\x12\x0f\n\tCANCELLED\x10\xa0\xb6\x18\x12\x12\n\x0cUNDETERMINED\x10\xaa\xb6\x18\x12\x11\n\x0bUNSUPPORTED\x10\xb4\xb6\x18\x12\x12\n\x0cSESSION_BUSY\x10\xbe\xb6\x18\x42Q\n\x08tech.ydbB\x11StatusCodesProtosZ2github.com/ydb-platform/ydb-go-genproto/protos/Ydbb\x06proto3'
)



_STATUSIDS_STATUSCODE = _descriptor.EnumDescriptor(
  name='StatusCode',
  full_name='Ydb.StatusIds.StatusCode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STATUS_CODE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=1, number=400000,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BAD_REQUEST', index=2, number=400010,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNAUTHORIZED', index=3, number=400020,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INTERNAL_ERROR', index=4, number=400030,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ABORTED', index=5, number=400040,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNAVAILABLE', index=6, number=400050,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='OVERLOADED', index=7, number=400060,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SCHEME_ERROR', index=8, number=400070,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GENERIC_ERROR', index=9, number=400080,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TIMEOUT', index=10, number=400090,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BAD_SESSION', index=11, number=400100,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PRECONDITION_FAILED', index=12, number=400120,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ALREADY_EXISTS', index=13, number=400130,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NOT_FOUND', index=14, number=400140,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SESSION_EXPIRED', index=15, number=400150,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CANCELLED', index=16, number=400160,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNDETERMINED', index=17, number=400170,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNSUPPORTED', index=18, number=400180,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SESSION_BUSY', index=19, number=400190,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=53,
  serialized_end=462,
)
_sym_db.RegisterEnumDescriptor(_STATUSIDS_STATUSCODE)


_STATUSIDS = _descriptor.Descriptor(
  name='StatusIds',
  full_name='Ydb.StatusIds',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _STATUSIDS_STATUSCODE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=39,
  serialized_end=462,
)

_STATUSIDS_STATUSCODE.containing_type = _STATUSIDS
DESCRIPTOR.message_types_by_name['StatusIds'] = _STATUSIDS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

StatusIds = _reflection.GeneratedProtocolMessageType('StatusIds', (_message.Message,), {
  'DESCRIPTOR' : _STATUSIDS,
  '__module__' : 'protos.ydb_status_codes_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.StatusIds)
  })
_sym_db.RegisterMessage(StatusIds)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb_auth_v1.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb._grpc.v4.protos import ydb_auth_pb2 as protos_dot_ydb__auth__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11ydb_auth_v1.proto\x12\x0bYdb.Auth.V1\x1a\x15protos/ydb_auth.proto2G\n\x0b\x41uthService\x12\x38\n\x05Login\x12\x16.Ydb.Auth.LoginRequest\x1a\x17.Ydb.Auth.LoginResponseBG\n\x10tech.ydb.auth.v1Z3github.com/ydb-platform/ydb-go-genproto/Ydb_Auth_V1b\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ydb_auth_v1_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\020tech.ydb.auth.v1Z3github.com/ydb-platform/ydb-go-genproto/Ydb_Auth_V1'
  _AUTHSERVICE._serialized_start=57
  _AUTHSERVICE._serialized_end=128
# @@protoc_insertion_point(module_scope)



import enum

from . import _apis

from . import settings_impl as s_impl

# Workaround for good IDE and universal for runtime
# noinspection PyUnreachableCode
if False:
    from ._grpc.v4.protos import ydb_import_pb2
    from ._grpc.v4 import ydb_import_v1_pb2_grpc
else:
    from ._grpc.common.protos import ydb_import_pb2
    from ._grpc.common import ydb_import_v1_pb2_grpc


from . import operation

_ImportFromS3 = "ImportFromS3"
_progresses = {}


@enum.unique
class ImportProgress(enum.IntEnum):
    UNSPECIFIED = 0
    PREPARING = 1
    TRANSFER_DATA = 2
    BUILD_INDEXES = 3
    DONE = 4
    CANCELLATION = 5
    CANCELLED = 6


def _initialize_progresses():
    for key, value in ydb_import_pb2.ImportProgress.Progress.items():
        _progresses[value] = getattr(ImportProgress, key[len("PROGRESS_") :])


_initialize_progresses()


class ImportFromS3Operation(operation.Operation):
    def __init__(self, rpc_state, response, driver):
        super(ImportFromS3Operation, self).__init__(rpc_state, response, driver)
        metadata = ydb_import_pb2.ImportFromS3Metadata()
        response.operation.metadata.Unpack(metadata)
        self.progress = _progresses.get(metadata.progress)

    def __str__(self):
        return "ImportFromS3Operation<id: %s, progress: %s>" % (
            self.id,
            self.progress.name,
        )

    def __repr__(self):
        return self.__str__()


class ImportFromS3Settings(s_impl.BaseRequestSettings):
    def __init__(self):
        super(ImportFromS3Settings, self).__init__()
        self.items = []
        self.bucket = None
        self.endpoint = None
        self.scheme = 2
        self.uid = None
        self.access_key = None
        self.secret_key = None
        self.number_of_retries = 0

    def with_scheme(self, scheme):
        self.scheme = scheme
        return self

    def with_bucket(self, bucket):
        self.bucket = bucket
        return self

    def with_endpoint(self, endpoint):
        self.endpoint = endpoint
        return self

    def with_access_key(self, access_key):
        self.access_key = access_key
        return self

    def with_uid(self, uid):
        self.uid = uid
        return self

    def with_secret_key(self, secret_key):
        self.secret_key = secret_key
        return self

    def with_number_of_retries(self, number_of_retries):
        self.number_of_retries = number_of_retries
        return self

    def with_source_and_destination(self, source_path, destination_prefix):
        return self.with_item((source_path, destination_prefix))

    def with_item(self, item):
        self.items.append(item)
        return self

    def with_items(self, *items):
        self.items.extend(items)
        return self


def _get_operation_request(operation_id):
    request = _apis.ydb_operation.GetOperationRequest(id=operation_id)
    return request


def _import_from_s3_request_factory(settings):
    request = ydb_import_pb2.ImportFromS3Request(
        settings=ydb_import_pb2.ImportFromS3Settings(
            endpoint=settings.endpoint,
            bucket=settings.bucket,
            access_key=settings.access_key,
            secret_key=settings.secret_key,
            scheme=settings.scheme,
        )
    )

    if settings.uid is not None:
        request.operation_params.labels["uid"] = settings.uid

    if settings.number_of_retries > 0:
        request.settings.number_of_retries = settings.number_of_retries

    for source, destination in settings.items:
        request.settings.items.add(
            source_prefix=source,
            destination_path=destination,
        )

    return request


class ImportClient(object):
    def __init__(self, driver):
        self._driver = driver

    def get_import_from_s3_operation(self, operation_id, settings=None):
        return self._driver(
            _get_operation_request(operation_id),
            _apis.OperationService.Stub,
            _apis.OperationService.GetOperation,
            ImportFromS3Operation,
            settings,
            (self._driver,),
        )

    def import_from_s3(self, settings):
        return self._driver(
            _import_from_s3_request_factory(settings),
            ydb_import_v1_pb2_grpc.ImportServiceStub,
            _ImportFromS3,
            ImportFromS3Operation,
            settings,
            (self._driver,),
        )



# -*- coding: utf-8 -*-
# Workaround for good IDE and universal for runtime
# noinspection PyUnreachableCode
if False:
    from ._grpc.v4 import (
        ydb_cms_v1_pb2_grpc,
        ydb_discovery_v1_pb2_grpc,
        ydb_scheme_v1_pb2_grpc,
        ydb_table_v1_pb2_grpc,
        ydb_operation_v1_pb2_grpc,
    )

    from ._grpc.v4.protos import (
        ydb_status_codes_pb2,
        ydb_discovery_pb2,
        ydb_scheme_pb2,
        ydb_table_pb2,
        ydb_value_pb2,
        ydb_operation_pb2,
        ydb_common_pb2,
    )
else:
    from ._grpc.common import (
        ydb_cms_v1_pb2_grpc,
        ydb_discovery_v1_pb2_grpc,
        ydb_scheme_v1_pb2_grpc,
        ydb_table_v1_pb2_grpc,
        ydb_operation_v1_pb2_grpc,
    )

    from ._grpc.common.protos import (
        ydb_status_codes_pb2,
        ydb_discovery_pb2,
        ydb_scheme_pb2,
        ydb_table_pb2,
        ydb_value_pb2,
        ydb_operation_pb2,
        ydb_common_pb2,
    )

StatusIds = ydb_status_codes_pb2.StatusIds
FeatureFlag = ydb_common_pb2.FeatureFlag
primitive_types = ydb_value_pb2.Type.PrimitiveTypeId
ydb_value = ydb_value_pb2
ydb_scheme = ydb_scheme_pb2
ydb_table = ydb_table_pb2
ydb_discovery = ydb_discovery_pb2
ydb_operation = ydb_operation_pb2


class CmsService(object):
    Stub = ydb_cms_v1_pb2_grpc.CmsServiceStub


class DiscoveryService(object):
    Stub = ydb_discovery_v1_pb2_grpc.DiscoveryServiceStub
    ListEndpoints = "ListEndpoints"


class OperationService(object):
    Stub = ydb_operation_v1_pb2_grpc.OperationServiceStub
    ForgetOperation = "ForgetOperation"
    GetOperation = "GetOperation"
    CancelOperation = "CancelOperation"


class SchemeService(object):
    Stub = ydb_scheme_v1_pb2_grpc.SchemeServiceStub
    MakeDirectory = "MakeDirectory"
    RemoveDirectory = "RemoveDirectory"
    ListDirectory = "ListDirectory"
    DescribePath = "DescribePath"
    ModifyPermissions = "ModifyPermissions"


class TableService(object):
    Stub = ydb_table_v1_pb2_grpc.TableServiceStub

    StreamExecuteScanQuery = "StreamExecuteScanQuery"
    ExplainDataQuery = "ExplainDataQuery"
    CreateTable = "CreateTable"
    DropTable = "DropTable"
    AlterTable = "AlterTable"
    CopyTables = "CopyTables"
    RenameTables = "RenameTables"
    DescribeTable = "DescribeTable"
    CreateSession = "CreateSession"
    DeleteSession = "DeleteSession"
    ExecuteSchemeQuery = "ExecuteSchemeQuery"
    PrepareDataQuery = "PrepareDataQuery"
    ExecuteDataQuery = "ExecuteDataQuery"
    BeginTransaction = "BeginTransaction"
    CommitTransaction = "CommitTransaction"
    RollbackTransaction = "RollbackTransaction"
    KeepAlive = "KeepAlive"
    StreamReadTable = "StreamReadTable"
    BulkUpsert = "BulkUpsert"



# -*- coding: utf-8 -*-
from . import credentials as credentials_impl, table, scheme, pool
from . import tracing
import six
import os
import grpc
from . import _utilities

if six.PY2:
    Any = None
else:
    from typing import Any  # noqa


class RPCCompression:
    """Indicates the compression method to be used for an RPC."""

    NoCompression = grpc.Compression.NoCompression
    Deflate = grpc.Compression.Deflate
    Gzip = grpc.Compression.Gzip


def default_credentials(credentials=None, tracer=None):
    tracer = tracer if tracer is not None else tracing.Tracer(None)
    with tracer.trace("Driver.default_credentials") as ctx:
        if credentials is not None:
            ctx.trace({"credentials.prepared": True})
            return credentials

        service_account_key_file = os.getenv("YDB_SERVICE_ACCOUNT_KEY_FILE_CREDENTIALS")
        if service_account_key_file is not None:
            ctx.trace({"credentials.service_account_key_file": True})
            import ydb.iam

            return ydb.iam.ServiceAccountCredentials.from_file(service_account_key_file)

        anonymous_credetials = os.getenv("YDB_ANONYMOUS_CREDENTIALS", "0") == "1"
        if anonymous_credetials:
            ctx.trace({"credentials.anonymous": True})
            return credentials_impl.AnonymousCredentials()

        metadata_credentials = os.getenv("YDB_METADATA_CREDENTIALS", "0") == "1"
        if metadata_credentials:
            ctx.trace({"credentials.metadata": True})
            import ydb.iam

            return ydb.iam.MetadataUrlCredentials(tracer=tracer)

        access_token = os.getenv("YDB_ACCESS_TOKEN_CREDENTIALS")
        if access_token is not None:
            ctx.trace({"credentials.access_token": True})
            return credentials_impl.AuthTokenCredentials(access_token)

        import ydb.iam

        return ydb.iam.MetadataUrlCredentials(tracer=tracer)


class DriverConfig(object):
    __slots__ = (
        "endpoint",
        "database",
        "ca_cert",
        "channel_options",
        "credentials",
        "use_all_nodes",
        "root_certificates",
        "certificate_chain",
        "private_key",
        "grpc_keep_alive_timeout",
        "secure_channel",
        "table_client_settings",
        "endpoints",
        "primary_user_agent",
        "tracer",
        "grpc_lb_policy_name",
        "discovery_request_timeout",
        "compression",
    )

    def __init__(
        self,
        endpoint,
        database=None,
        ca_cert=None,
        auth_token=None,
        channel_options=None,
        credentials=None,
        use_all_nodes=False,
        root_certificates=None,
        certificate_chain=None,
        private_key=None,
        grpc_keep_alive_timeout=None,
        table_client_settings=None,
        endpoints=None,
        primary_user_agent="python-library",
        tracer=None,
        grpc_lb_policy_name="round_robin",
        discovery_request_timeout=10,
        compression=None,
    ):
        """
        A driver config to initialize a driver instance

        :param endpoint: A endpoint specified in pattern host:port to be used for initial channel initialization and for YDB endpoint discovery mechanism
        :param database: A name of the database
        :param ca_cert: A CA certificate when SSL should be used
        :param auth_token: A authentication token
        :param credentials: An instance of AbstractCredentials
        :param use_all_nodes: A balancing policy that forces to use all available nodes.
        :param root_certificates: The PEM-encoded root certificates as a byte string.
        :param private_key: The PEM-encoded private key as a byte string, or None if no\
        private key should be used.
        :param certificate_chain: The PEM-encoded certificate chain as a byte string\
        to use or or None if no certificate chain should be used.
        :param grpc_keep_alive_timeout: GRpc KeepAlive timeout, ms
        :param ydb.Tracer tracer: ydb.Tracer instance to trace requests in driver.\
        If tracing aio ScopeManager must be ContextVarsScopeManager
        :param grpc_lb_policy_name: A load balancing policy to be used for discovery channel construction. Default value is `round_round`
        :param discovery_request_timeout: A default timeout to complete the discovery. The default value is 10 seconds.

        """
        self.endpoint = endpoint
        self.database = database
        self.ca_cert = ca_cert
        self.channel_options = channel_options
        self.secure_channel = _utilities.is_secure_protocol(endpoint)
        self.endpoint = _utilities.wrap_endpoint(self.endpoint)
        self.endpoints = []
        if endpoints is not None:
            self.endpoints = [_utilities.wrap_endpoint(endp) for endp in endpoints]
        if auth_token is not None:
            credentials = credentials_impl.AuthTokenCredentials(auth_token)
        self.credentials = credentials
        self.use_all_nodes = use_all_nodes
        self.root_certificates = root_certificates
        self.certificate_chain = certificate_chain
        self.private_key = private_key
        self.grpc_keep_alive_timeout = grpc_keep_alive_timeout
        self.table_client_settings = table_client_settings
        self.primary_user_agent = primary_user_agent
        self.tracer = tracer if tracer is not None else tracing.Tracer(None)
        self.grpc_lb_policy_name = grpc_lb_policy_name
        self.discovery_request_timeout = discovery_request_timeout
        self.compression = compression

    def set_database(self, database):
        self.database = database
        return self

    @classmethod
    def default_from_endpoint_and_database(
        cls, endpoint, database, root_certificates=None, credentials=None, **kwargs
    ):
        return cls(
            endpoint,
            database,
            credentials=default_credentials(credentials),
            root_certificates=root_certificates,
            **kwargs
        )

    @classmethod
    def default_from_connection_string(
        cls, connection_string, root_certificates=None, credentials=None, **kwargs
    ):
        endpoint, database = _utilities.parse_connection_string(connection_string)
        return cls(
            endpoint,
            database,
            credentials=default_credentials(credentials),
            root_certificates=root_certificates,
            **kwargs
        )

    def set_grpc_keep_alive_timeout(self, timeout):
        self.grpc_keep_alive_timeout = timeout
        return self


ConnectionParams = DriverConfig


def get_config(
    driver_config=None,
    connection_string=None,
    endpoint=None,
    database=None,
    root_certificates=None,
    credentials=None,
    config_class=DriverConfig,
    **kwargs
):
    if driver_config is None:
        if connection_string is not None:
            driver_config = config_class.default_from_connection_string(
                connection_string, root_certificates, credentials, **kwargs
            )
        else:
            driver_config = config_class.default_from_endpoint_and_database(
                endpoint, database, root_certificates, credentials, **kwargs
            )
        return driver_config
    return driver_config


class Driver(pool.ConnectionPool):
    __slots__ = ("scheme_client", "table_client")

    def __init__(
        self,
        driver_config=None,
        connection_string=None,
        endpoint=None,
        database=None,
        root_certificates=None,
        credentials=None,
        **kwargs
    ):
        """
        Constructs a driver instance to be used in table and scheme clients.
        It encapsulates endpoints discovery mechanism and provides ability to execute RPCs
        on discovered endpoints

        :param driver_config: A driver config
        :param connection_string: A string in the following format: <protocol>://<hostame>:<port>/?database=/path/to/the/database
        :param endpoint: An endpoint specified in the following format: <protocol>://<hostame>:<port>
        :param database: A database path
        :param credentials: A credentials. If not specifed credentials constructed by default.
        """
        driver_config = get_config(
            driver_config,
            connection_string,
            endpoint,
            database,
            root_certificates,
            credentials,
        )

        super(Driver, self).__init__(driver_config)
        self.scheme_client = scheme.SchemeClient(self)
        self.table_client = table.TableClient(self, driver_config.table_client_settings)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from ydb._grpc.v3.protos import ydb_topic_pb2 as protos_dot_ydb__topic__pb2


class TopicServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.StreamWrite = channel.stream_stream(
                '/Ydb.Topic.V1.TopicService/StreamWrite',
                request_serializer=protos_dot_ydb__topic__pb2.StreamWriteMessage.FromClient.SerializeToString,
                response_deserializer=protos_dot_ydb__topic__pb2.StreamWriteMessage.FromServer.FromString,
                )
        self.StreamRead = channel.stream_stream(
                '/Ydb.Topic.V1.TopicService/StreamRead',
                request_serializer=protos_dot_ydb__topic__pb2.StreamReadMessage.FromClient.SerializeToString,
                response_deserializer=protos_dot_ydb__topic__pb2.StreamReadMessage.FromServer.FromString,
                )
        self.CreateTopic = channel.unary_unary(
                '/Ydb.Topic.V1.TopicService/CreateTopic',
                request_serializer=protos_dot_ydb__topic__pb2.CreateTopicRequest.SerializeToString,
                response_deserializer=protos_dot_ydb__topic__pb2.CreateTopicResponse.FromString,
                )
        self.DescribeTopic = channel.unary_unary(
                '/Ydb.Topic.V1.TopicService/DescribeTopic',
                request_serializer=protos_dot_ydb__topic__pb2.DescribeTopicRequest.SerializeToString,
                response_deserializer=protos_dot_ydb__topic__pb2.DescribeTopicResponse.FromString,
                )
        self.DescribeConsumer = channel.unary_unary(
                '/Ydb.Topic.V1.TopicService/DescribeConsumer',
                request_serializer=protos_dot_ydb__topic__pb2.DescribeConsumerRequest.SerializeToString,
                response_deserializer=protos_dot_ydb__topic__pb2.DescribeConsumerResponse.FromString,
                )
        self.AlterTopic = channel.unary_unary(
                '/Ydb.Topic.V1.TopicService/AlterTopic',
                request_serializer=protos_dot_ydb__topic__pb2.AlterTopicRequest.SerializeToString,
                response_deserializer=protos_dot_ydb__topic__pb2.AlterTopicResponse.FromString,
                )
        self.DropTopic = channel.unary_unary(
                '/Ydb.Topic.V1.TopicService/DropTopic',
                request_serializer=protos_dot_ydb__topic__pb2.DropTopicRequest.SerializeToString,
                response_deserializer=protos_dot_ydb__topic__pb2.DropTopicResponse.FromString,
                )


class TopicServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def StreamWrite(self, request_iterator, context):
        """Create Write Session
        Pipeline example:
        client                  server
        InitRequest(Topic, MessageGroupID, ...)
        ---------------->
        InitResponse(Partition, MaxSeqNo, ...)
        <----------------
        WriteRequest(data1, seqNo1)
        ---------------->
        WriteRequest(data2, seqNo2)
        ---------------->
        WriteResponse(seqNo1, offset1, ...)
        <----------------
        WriteRequest(data3, seqNo3)
        ---------------->
        WriteResponse(seqNo2, offset2, ...)
        <----------------
        [something went wrong] (status != SUCCESS, issues not empty)
        <----------------
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamRead(self, request_iterator, context):
        """Create Read Session
        Pipeline:
        client                  server
        InitRequest(Topics, ClientId, ...)
        ---------------->
        InitResponse(SessionId)
        <----------------
        ReadRequest
        ---------------->
        ReadRequest
        ---------------->
        StartPartitionSessionRequest(Topic1, Partition1, PartitionSessionID1, ...)
        <----------------
        StartPartitionSessionRequest(Topic2, Partition2, PartitionSessionID2, ...)
        <----------------
        StartPartitionSessionResponse(PartitionSessionID1, ...)
        client must respond with this message to actually start recieving data messages from this partition
        ---------------->
        StopPartitionSessionRequest(PartitionSessionID1, ...)
        <----------------
        StopPartitionSessionResponse(PartitionSessionID1, ...)
        only after this response server will give this parittion to other session.
        ---------------->
        StartPartitionSessionResponse(PartitionSession2, ...)
        ---------------->
        ReadResponse(data, ...)
        <----------------
        CommitRequest(PartitionCommit1, ...)
        ---------------->
        CommitResponse(PartitionCommitAck1, ...)
        <----------------
        [something went wrong] (status != SUCCESS, issues not empty)
        <----------------
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTopic(self, request, context):
        """Create topic command.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeTopic(self, request, context):
        """Describe topic command.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeConsumer(self, request, context):
        """Describe topic's consumer command.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AlterTopic(self, request, context):
        """Alter topic command.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DropTopic(self, request, context):
        """Drop topic command.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TopicServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'StreamWrite': grpc.stream_stream_rpc_method_handler(
                    servicer.StreamWrite,
                    request_deserializer=protos_dot_ydb__topic__pb2.StreamWriteMessage.FromClient.FromString,
                    response_serializer=protos_dot_ydb__topic__pb2.StreamWriteMessage.FromServer.SerializeToString,
            ),
            'StreamRead': grpc.stream_stream_rpc_method_handler(
                    servicer.StreamRead,
                    request_deserializer=protos_dot_ydb__topic__pb2.StreamReadMessage.FromClient.FromString,
                    response_serializer=protos_dot_ydb__topic__pb2.StreamReadMessage.FromServer.SerializeToString,
            ),
            'CreateTopic': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTopic,
                    request_deserializer=protos_dot_ydb__topic__pb2.CreateTopicRequest.FromString,
                    response_serializer=protos_dot_ydb__topic__pb2.CreateTopicResponse.SerializeToString,
            ),
            'DescribeTopic': grpc.unary_unary_rpc_method_handler(
                    servicer.DescribeTopic,
                    request_deserializer=protos_dot_ydb__topic__pb2.DescribeTopicRequest.FromString,
                    response_serializer=protos_dot_ydb__topic__pb2.DescribeTopicResponse.SerializeToString,
            ),
            'DescribeConsumer': grpc.unary_unary_rpc_method_handler(
                    servicer.DescribeConsumer,
                    request_deserializer=protos_dot_ydb__topic__pb2.DescribeConsumerRequest.FromString,
                    response_serializer=protos_dot_ydb__topic__pb2.DescribeConsumerResponse.SerializeToString,
            ),
            'AlterTopic': grpc.unary_unary_rpc_method_handler(
                    servicer.AlterTopic,
                    request_deserializer=protos_dot_ydb__topic__pb2.AlterTopicRequest.FromString,
                    response_serializer=protos_dot_ydb__topic__pb2.AlterTopicResponse.SerializeToString,
            ),
            'DropTopic': grpc.unary_unary_rpc_method_handler(
                    servicer.DropTopic,
                    request_deserializer=protos_dot_ydb__topic__pb2.DropTopicRequest.FromString,
                    response_serializer=protos_dot_ydb__topic__pb2.DropTopicResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'Ydb.Topic.V1.TopicService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class TopicService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def StreamWrite(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/Ydb.Topic.V1.TopicService/StreamWrite',
            protos_dot_ydb__topic__pb2.StreamWriteMessage.FromClient.SerializeToString,
            protos_dot_ydb__topic__pb2.StreamWriteMessage.FromServer.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamRead(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/Ydb.Topic.V1.TopicService/StreamRead',
            protos_dot_ydb__topic__pb2.StreamReadMessage.FromClient.SerializeToString,
            protos_dot_ydb__topic__pb2.StreamReadMessage.FromServer.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTopic(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Ydb.Topic.V1.TopicService/CreateTopic',
            protos_dot_ydb__topic__pb2.CreateTopicRequest.SerializeToString,
            protos_dot_ydb__topic__pb2.CreateTopicResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DescribeTopic(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Ydb.Topic.V1.TopicService/DescribeTopic',
            protos_dot_ydb__topic__pb2.DescribeTopicRequest.SerializeToString,
            protos_dot_ydb__topic__pb2.DescribeTopicResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DescribeConsumer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Ydb.Topic.V1.TopicService/DescribeConsumer',
            protos_dot_ydb__topic__pb2.DescribeConsumerRequest.SerializeToString,
            protos_dot_ydb__topic__pb2.DescribeConsumerResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AlterTopic(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Ydb.Topic.V1.TopicService/AlterTopic',
            protos_dot_ydb__topic__pb2.AlterTopicRequest.SerializeToString,
            protos_dot_ydb__topic__pb2.AlterTopicResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DropTopic(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Ydb.Topic.V1.TopicService/DropTopic',
            protos_dot_ydb__topic__pb2.DropTopicRequest.SerializeToString,
            protos_dot_ydb__topic__pb2.DropTopicResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/ydb_discovery.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb._grpc.v4.protos import ydb_operation_pb2 as protos_dot_ydb__operation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1aprotos/ydb_discovery.proto\x12\rYdb.Discovery\x1a\x1aprotos/ydb_operation.proto\"9\n\x14ListEndpointsRequest\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\x12\x0f\n\x07service\x18\x02 \x03(\t\"\xc3\x01\n\x0c\x45ndpointInfo\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\r\x12\x13\n\x0bload_factor\x18\x03 \x01(\x02\x12\x0b\n\x03ssl\x18\x04 \x01(\x08\x12\x0f\n\x07service\x18\x05 \x03(\t\x12\x10\n\x08location\x18\x06 \x01(\t\x12\x0f\n\x07node_id\x18\x07 \x01(\r\x12\r\n\x05ip_v4\x18\x08 \x03(\t\x12\r\n\x05ip_v6\x18\t \x03(\t\x12 \n\x18ssl_target_name_override\x18\n \x01(\t\"\\\n\x13ListEndpointsResult\x12.\n\tendpoints\x18\x01 \x03(\x0b\x32\x1b.Ydb.Discovery.EndpointInfo\x12\x15\n\rself_location\x18\x02 \x01(\t\"E\n\x15ListEndpointsResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\'\n\rWhoAmIRequest\x12\x16\n\x0einclude_groups\x18\x01 \x01(\x08\",\n\x0cWhoAmIResult\x12\x0c\n\x04user\x18\x01 \x01(\t\x12\x0e\n\x06groups\x18\x02 \x03(\t\">\n\x0eWhoAmIResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.OperationBf\n\x12tech.ydb.discoveryB\x0f\x44iscoveryProtosZ<github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Discovery\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protos.ydb_discovery_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\022tech.ydb.discoveryB\017DiscoveryProtosZ<github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Discovery\370\001\001'
  _LISTENDPOINTSREQUEST._serialized_start=73
  _LISTENDPOINTSREQUEST._serialized_end=130
  _ENDPOINTINFO._serialized_start=133
  _ENDPOINTINFO._serialized_end=328
  _LISTENDPOINTSRESULT._serialized_start=330
  _LISTENDPOINTSRESULT._serialized_end=422
  _LISTENDPOINTSRESPONSE._serialized_start=424
  _LISTENDPOINTSRESPONSE._serialized_end=493
  _WHOAMIREQUEST._serialized_start=495
  _WHOAMIREQUEST._serialized_end=534
  _WHOAMIRESULT._serialized_start=536
  _WHOAMIRESULT._serialized_end=580
  _WHOAMIRESPONSE._serialized_start=582
  _WHOAMIRESPONSE._serialized_end=644
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ydb_cms_v1.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb._grpc.v3.protos import ydb_cms_pb2 as protos_dot_ydb__cms__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ydb_cms_v1.proto',
  package='Ydb.Cms.V1',
  syntax='proto3',
  serialized_options=b'\n\017tech.ydb.cms.v1Z2github.com/ydb-platform/ydb-go-genproto/Ydb_Cms_V1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x10ydb_cms_v1.proto\x12\nYdb.Cms.V1\x1a\x14protos/ydb_cms.proto2\x9c\x04\n\nCmsService\x12Q\n\x0e\x43reateDatabase\x12\x1e.Ydb.Cms.CreateDatabaseRequest\x1a\x1f.Ydb.Cms.CreateDatabaseResponse\x12Z\n\x11GetDatabaseStatus\x12!.Ydb.Cms.GetDatabaseStatusRequest\x1a\".Ydb.Cms.GetDatabaseStatusResponse\x12N\n\rAlterDatabase\x12\x1d.Ydb.Cms.AlterDatabaseRequest\x1a\x1e.Ydb.Cms.AlterDatabaseResponse\x12N\n\rListDatabases\x12\x1d.Ydb.Cms.ListDatabasesRequest\x1a\x1e.Ydb.Cms.ListDatabasesResponse\x12Q\n\x0eRemoveDatabase\x12\x1e.Ydb.Cms.RemoveDatabaseRequest\x1a\x1f.Ydb.Cms.RemoveDatabaseResponse\x12l\n\x17\x44\x65scribeDatabaseOptions\x12\'.Ydb.Cms.DescribeDatabaseOptionsRequest\x1a(.Ydb.Cms.DescribeDatabaseOptionsResponseBE\n\x0ftech.ydb.cms.v1Z2github.com/ydb-platform/ydb-go-genproto/Ydb_Cms_V1b\x06proto3'
  ,
  dependencies=[protos_dot_ydb__cms__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)


DESCRIPTOR._options = None

_CMSSERVICE = _descriptor.ServiceDescriptor(
  name='CmsService',
  full_name='Ydb.Cms.V1.CmsService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=55,
  serialized_end=595,
  methods=[
  _descriptor.MethodDescriptor(
    name='CreateDatabase',
    full_name='Ydb.Cms.V1.CmsService.CreateDatabase',
    index=0,
    containing_service=None,
    input_type=protos_dot_ydb__cms__pb2._CREATEDATABASEREQUEST,
    output_type=protos_dot_ydb__cms__pb2._CREATEDATABASERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDatabaseStatus',
    full_name='Ydb.Cms.V1.CmsService.GetDatabaseStatus',
    index=1,
    containing_service=None,
    input_type=protos_dot_ydb__cms__pb2._GETDATABASESTATUSREQUEST,
    output_type=protos_dot_ydb__cms__pb2._GETDATABASESTATUSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='AlterDatabase',
    full_name='Ydb.Cms.V1.CmsService.AlterDatabase',
    index=2,
    containing_service=None,
    input_type=protos_dot_ydb__cms__pb2._ALTERDATABASEREQUEST,
    output_type=protos_dot_ydb__cms__pb2._ALTERDATABASERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListDatabases',
    full_name='Ydb.Cms.V1.CmsService.ListDatabases',
    index=3,
    containing_service=None,
    input_type=protos_dot_ydb__cms__pb2._LISTDATABASESREQUEST,
    output_type=protos_dot_ydb__cms__pb2._LISTDATABASESRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RemoveDatabase',
    full_name='Ydb.Cms.V1.CmsService.RemoveDatabase',
    index=4,
    containing_service=None,
    input_type=protos_dot_ydb__cms__pb2._REMOVEDATABASEREQUEST,
    output_type=protos_dot_ydb__cms__pb2._REMOVEDATABASERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DescribeDatabaseOptions',
    full_name='Ydb.Cms.V1.CmsService.DescribeDatabaseOptions',
    index=5,
    containing_service=None,
    input_type=protos_dot_ydb__cms__pb2._DESCRIBEDATABASEOPTIONSREQUEST,
    output_type=protos_dot_ydb__cms__pb2._DESCRIBEDATABASEOPTIONSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_CMSSERVICE)

DESCRIPTOR.services_by_name['CmsService'] = _CMSSERVICE

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
import abc
import six
from . import tracing, issues, connection
from . import settings as settings_impl
import threading
from concurrent import futures
import logging
import time

# Workaround for good IDE and universal for runtime
# noinspection PyUnreachableCode
if False:
    from ._grpc.v4.protos import ydb_auth_pb2
    from ._grpc.v4 import ydb_auth_v1_pb2_grpc
else:
    from ._grpc.common.protos import ydb_auth_pb2
    from ._grpc.common import ydb_auth_v1_pb2_grpc


YDB_AUTH_TICKET_HEADER = "x-ydb-auth-ticket"
logger = logging.getLogger(__name__)


@six.add_metaclass(abc.ABCMeta)
class AbstractCredentials(object):
    """
    An abstract class that provides auth metadata
    """


@six.add_metaclass(abc.ABCMeta)
class Credentials(object):
    def __init__(self, tracer=None):
        self.tracer = tracer if tracer is not None else tracing.Tracer(None)

    @abc.abstractmethod
    def auth_metadata(self):
        """
        :return: An iterable with auth metadata
        """
        pass


class OneToManyValue(object):
    def __init__(self):
        self._value = None
        self._condition = threading.Condition()

    def consume(self, timeout=3):
        with self._condition:
            if self._value is None:
                self._condition.wait(timeout=timeout)
            return self._value

    def update(self, n_value):
        with self._condition:
            prev_value = self._value
            self._value = n_value
            if prev_value is None:
                self._condition.notify_all()


class AtMostOneExecution(object):
    def __init__(self):
        self._can_schedule = True
        self._lock = threading.Lock()
        self._tp = futures.ThreadPoolExecutor(1)

    def wrapped_execution(self, callback):
        try:
            callback()
        except Exception:
            pass

        finally:
            self.cleanup()

    def submit(self, callback):
        with self._lock:
            if self._can_schedule:
                self._tp.submit(self.wrapped_execution, callback)
                self._can_schedule = False

    def cleanup(self):
        with self._lock:
            self._can_schedule = True


@six.add_metaclass(abc.ABCMeta)
class AbstractExpiringTokenCredentials(Credentials):
    def __init__(self, tracer=None):
        super(AbstractExpiringTokenCredentials, self).__init__(tracer)
        self._expires_in = 0
        self._refresh_in = 0
        self._hour = 60 * 60
        self._cached_token = OneToManyValue()
        self._tp = AtMostOneExecution()
        self.logger = logger.getChild(self.__class__.__name__)
        self.last_error = None
        self.extra_error_message = ""

    @abc.abstractmethod
    def _make_token_request(self):
        pass

    def _log_refresh_start(self, current_time):
        self.logger.debug("Start refresh token from metadata")
        if current_time > self._refresh_in:
            self.logger.info(
                "Cached token reached refresh_in deadline, current time %s, deadline %s",
                current_time,
                self._refresh_in,
            )

        if current_time > self._expires_in and self._expires_in > 0:
            self.logger.error(
                "Cached token reached expires_in deadline, current time %s, deadline %s",
                current_time,
                self._expires_in,
            )

    def _update_expiration_info(self, auth_metadata):
        self._expires_in = time.time() + min(
            self._hour, auth_metadata["expires_in"] / 2
        )
        self._refresh_in = time.time() + min(
            self._hour / 2, auth_metadata["expires_in"] / 4
        )

    def _refresh(self):
        current_time = time.time()
        self._log_refresh_start(current_time)
        try:
            token_response = self._make_token_request()
            self._cached_token.update(token_response["access_token"])
            self._update_expiration_info(token_response)
            self.logger.info(
                "Token refresh successful. current_time %s, refresh_in %s",
                current_time,
                self._refresh_in,
            )

        except (KeyboardInterrupt, SystemExit):
            return

        except Exception as e:
            self.last_error = str(e)
            time.sleep(1)
            self._tp.submit(self._refresh)

    @property
    @tracing.with_trace()
    def token(self):
        current_time = time.time()
        if current_time > self._refresh_in:
            tracing.trace(self.tracer, {"refresh": True})
            self._tp.submit(self._refresh)
        cached_token = self._cached_token.consume(timeout=3)
        tracing.trace(self.tracer, {"consumed": True})
        if cached_token is None:
            if self.last_error is None:
                raise issues.ConnectionError(
                    "%s: timeout occurred while waiting for token.\n%s"
                    % (
                        self.__class__.__name__,
                        self.extra_error_message,
                    )
                )
            raise issues.ConnectionError(
                "%s: %s.\n%s"
                % (self.__class__.__name__, self.last_error, self.extra_error_message)
            )
        return cached_token

    def auth_metadata(self):
        return [(YDB_AUTH_TICKET_HEADER, self.token)]


def _wrap_static_credentials_response(rpc_state, response):
    issues._process_response(response.operation)
    result = ydb_auth_pb2.LoginResult()
    response.operation.result.Unpack(result)
    return result


class StaticCredentials(AbstractExpiringTokenCredentials):
    def __init__(self, driver_config, user, password="", tracer=None):
        super(StaticCredentials, self).__init__(tracer)
        self.driver_config = driver_config
        self.user = user
        self.password = password
        self.request_timeout = 10

    def _make_token_request(self):
        conn = connection.Connection.ready_factory(
            self.driver_config.endpoint, self.driver_config
        )
        assert conn is not None, (
            "Failed to establish connection in to %s" % self.driver_config.endpoint
        )
        try:
            result = conn(
                ydb_auth_pb2.LoginRequest(user=self.user, password=self.password),
                ydb_auth_v1_pb2_grpc.AuthServiceStub,
                "Login",
                _wrap_static_credentials_response,
                settings_impl.BaseRequestSettings()
                .with_timeout(self.request_timeout)
                .with_need_rpc_auth(False),
            )
        finally:
            conn.close()
        return {"expires_in": 30 * 60, "access_token": result.token}


class AnonymousCredentials(Credentials):
    @staticmethod
    def auth_metadata():
        return []


class AuthTokenCredentials(Credentials):
    def __init__(self, token):
        self._token = token

    def auth_metadata(self):
        return [(YDB_AUTH_TICKET_HEADER, self._token)]


class AccessTokenCredentials(Credentials):
    def __init__(self, token):
        self._token = token

    def auth_metadata(self):
        return [(YDB_AUTH_TICKET_HEADER, self._token)]



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/ydb_value.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16protos/ydb_value.proto\x12\x03Ydb\x1a\x1cgoogle/protobuf/struct.proto\"/\n\x0b\x44\x65\x63imalType\x12\x11\n\tprecision\x18\x01 \x01(\r\x12\r\n\x05scale\x18\x02 \x01(\r\"\'\n\x0cOptionalType\x12\x17\n\x04item\x18\x01 \x01(\x0b\x32\t.Ydb.Type\"#\n\x08ListType\x12\x17\n\x04item\x18\x01 \x01(\x0b\x32\t.Ydb.Type\"e\n\x0bVariantType\x12%\n\x0btuple_items\x18\x01 \x01(\x0b\x32\x0e.Ydb.TupleTypeH\x00\x12\'\n\x0cstruct_items\x18\x02 \x01(\x0b\x32\x0f.Ydb.StructTypeH\x00\x42\x06\n\x04type\"(\n\tTupleType\x12\x1b\n\x08\x65lements\x18\x01 \x03(\x0b\x32\t.Ydb.Type\"5\n\x0cStructMember\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x17\n\x04type\x18\x02 \x01(\x0b\x32\t.Ydb.Type\"0\n\nStructType\x12\"\n\x07members\x18\x01 \x03(\x0b\x32\x11.Ydb.StructMember\">\n\x08\x44ictType\x12\x16\n\x03key\x18\x01 \x01(\x0b\x32\t.Ydb.Type\x12\x1a\n\x07payload\x18\x02 \x01(\x0b\x32\t.Ydb.Type\"2\n\nTaggedType\x12\x0b\n\x03tag\x18\x01 \x01(\t\x12\x17\n\x04type\x18\x02 \x01(\x0b\x32\t.Ydb.Type\"5\n\x06PgType\x12\x0b\n\x03oid\x18\x01 \x01(\r\x12\x0e\n\x06typlen\x18\x02 \x01(\x05\x12\x0e\n\x06typmod\x18\x03 \x01(\x05\"\xe2\x07\n\x04Type\x12,\n\x07type_id\x18\x01 \x01(\x0e\x32\x19.Ydb.Type.PrimitiveTypeIdH\x00\x12(\n\x0c\x64\x65\x63imal_type\x18\x02 \x01(\x0b\x32\x10.Ydb.DecimalTypeH\x00\x12*\n\roptional_type\x18\x65 \x01(\x0b\x32\x11.Ydb.OptionalTypeH\x00\x12\"\n\tlist_type\x18\x66 \x01(\x0b\x32\r.Ydb.ListTypeH\x00\x12$\n\ntuple_type\x18g \x01(\x0b\x32\x0e.Ydb.TupleTypeH\x00\x12&\n\x0bstruct_type\x18h \x01(\x0b\x32\x0f.Ydb.StructTypeH\x00\x12\"\n\tdict_type\x18i \x01(\x0b\x32\r.Ydb.DictTypeH\x00\x12(\n\x0cvariant_type\x18j \x01(\x0b\x32\x10.Ydb.VariantTypeH\x00\x12&\n\x0btagged_type\x18k \x01(\x0b\x32\x0f.Ydb.TaggedTypeH\x00\x12\x30\n\tvoid_type\x18\xc9\x01 \x01(\x0e\x32\x1a.google.protobuf.NullValueH\x00\x12\x30\n\tnull_type\x18\xca\x01 \x01(\x0e\x32\x1a.google.protobuf.NullValueH\x00\x12\x36\n\x0f\x65mpty_list_type\x18\xcb\x01 \x01(\x0e\x32\x1a.google.protobuf.NullValueH\x00\x12\x36\n\x0f\x65mpty_dict_type\x18\xcc\x01 \x01(\x0e\x32\x1a.google.protobuf.NullValueH\x00\x12\x1f\n\x07pg_type\x18\xcd\x01 \x01(\x0b\x32\x0b.Ydb.PgTypeH\x00\"\xf0\x02\n\x0fPrimitiveTypeId\x12!\n\x1dPRIMITIVE_TYPE_ID_UNSPECIFIED\x10\x00\x12\x08\n\x04\x42OOL\x10\x06\x12\x08\n\x04INT8\x10\x07\x12\t\n\x05UINT8\x10\x05\x12\t\n\x05INT16\x10\x08\x12\n\n\x06UINT16\x10\t\x12\t\n\x05INT32\x10\x01\x12\n\n\x06UINT32\x10\x02\x12\t\n\x05INT64\x10\x03\x12\n\n\x06UINT64\x10\x04\x12\t\n\x05\x46LOAT\x10!\x12\n\n\x06\x44OUBLE\x10 \x12\x08\n\x04\x44\x41TE\x10\x30\x12\x0c\n\x08\x44\x41TETIME\x10\x31\x12\r\n\tTIMESTAMP\x10\x32\x12\x0c\n\x08INTERVAL\x10\x33\x12\x0b\n\x07TZ_DATE\x10\x34\x12\x0f\n\x0bTZ_DATETIME\x10\x35\x12\x10\n\x0cTZ_TIMESTAMP\x10\x36\x12\x0b\n\x06STRING\x10\x81 \x12\t\n\x04UTF8\x10\x80$\x12\t\n\x04YSON\x10\x81$\x12\t\n\x04JSON\x10\x82$\x12\t\n\x04UUID\x10\x83$\x12\x12\n\rJSON_DOCUMENT\x10\x84$\x12\r\n\x08\x44YNUMBER\x10\x82&B\x06\n\x04type\"A\n\tValuePair\x12\x17\n\x03key\x18\x01 \x01(\x0b\x32\n.Ydb.Value\x12\x1b\n\x07payload\x18\x02 \x01(\x0b\x32\n.Ydb.Value\"\xb1\x03\n\x05Value\x12\x14\n\nbool_value\x18\x01 \x01(\x08H\x00\x12\x15\n\x0bint32_value\x18\x02 \x01(\x0fH\x00\x12\x16\n\x0cuint32_value\x18\x03 \x01(\x07H\x00\x12\x15\n\x0bint64_value\x18\x04 \x01(\x10H\x00\x12\x16\n\x0cuint64_value\x18\x05 \x01(\x06H\x00\x12\x15\n\x0b\x66loat_value\x18\x06 \x01(\x02H\x00\x12\x16\n\x0c\x64ouble_value\x18\x07 \x01(\x01H\x00\x12\x15\n\x0b\x62ytes_value\x18\x08 \x01(\x0cH\x00\x12\x14\n\ntext_value\x18\t \x01(\tH\x00\x12\x35\n\x0fnull_flag_value\x18\n \x01(\x0e\x32\x1a.google.protobuf.NullValueH\x00\x12\"\n\x0cnested_value\x18\x0b \x01(\x0b\x32\n.Ydb.ValueH\x00\x12\x11\n\x07low_128\x18\x0f \x01(\x06H\x00\x12\x19\n\x05items\x18\x0c \x03(\x0b\x32\n.Ydb.Value\x12\x1d\n\x05pairs\x18\r \x03(\x0b\x32\x0e.Ydb.ValuePair\x12\x15\n\rvariant_index\x18\x0e \x01(\r\x12\x10\n\x08high_128\x18\x10 \x01(\x06\x42\x07\n\x05value\"@\n\nTypedValue\x12\x17\n\x04type\x18\x01 \x01(\x0b\x32\t.Ydb.Type\x12\x19\n\x05value\x18\x02 \x01(\x0b\x32\n.Ydb.Value\"/\n\x06\x43olumn\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x17\n\x04type\x18\x02 \x01(\x0b\x32\t.Ydb.Type\"V\n\tResultSet\x12\x1c\n\x07\x63olumns\x18\x01 \x03(\x0b\x32\x0b.Ydb.Column\x12\x18\n\x04rows\x18\x02 \x03(\x0b\x32\n.Ydb.Value\x12\x11\n\ttruncated\x18\x03 \x01(\x08\x42N\n\x08tech.ydbB\x0bValueProtosZ2github.com/ydb-platform/ydb-go-genproto/protos/Ydb\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protos.ydb_value_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\010tech.ydbB\013ValueProtosZ2github.com/ydb-platform/ydb-go-genproto/protos/Ydb\370\001\001'
  _DECIMALTYPE._serialized_start=61
  _DECIMALTYPE._serialized_end=108
  _OPTIONALTYPE._serialized_start=110
  _OPTIONALTYPE._serialized_end=149
  _LISTTYPE._serialized_start=151
  _LISTTYPE._serialized_end=186
  _VARIANTTYPE._serialized_start=188
  _VARIANTTYPE._serialized_end=289
  _TUPLETYPE._serialized_start=291
  _TUPLETYPE._serialized_end=331
  _STRUCTMEMBER._serialized_start=333
  _STRUCTMEMBER._serialized_end=386
  _STRUCTTYPE._serialized_start=388
  _STRUCTTYPE._serialized_end=436
  _DICTTYPE._serialized_start=438
  _DICTTYPE._serialized_end=500
  _TAGGEDTYPE._serialized_start=502
  _TAGGEDTYPE._serialized_end=552
  _PGTYPE._serialized_start=554
  _PGTYPE._serialized_end=607
  _TYPE._serialized_start=610
  _TYPE._serialized_end=1604
  _TYPE_PRIMITIVETYPEID._serialized_start=1228
  _TYPE_PRIMITIVETYPEID._serialized_end=1596
  _VALUEPAIR._serialized_start=1606
  _VALUEPAIR._serialized_end=1671
  _VALUE._serialized_start=1674
  _VALUE._serialized_end=2107
  _TYPEDVALUE._serialized_start=2109
  _TYPEDVALUE._serialized_end=2173
  _COLUMN._serialized_start=2175
  _COLUMN._serialized_end=2222
  _RESULTSET._serialized_start=2224
  _RESULTSET._serialized_end=2310
# @@protoc_insertion_point(module_scope)



from ydb._grpc.common import *  # noqa
import sys
import warnings

sys.modules["ydb.public.api.grpc"] = sys.modules["ydb._grpc.common"]
warnings.warn(
    "using ydb.public.api.grpc module is deprecated. Don't use direct grpc dependencies."
)



from . import convert
from . import table


def global_allow_truncated_result(enabled: bool = True):
    """
    call global_allow_truncated_result(False) for more safe execution and compatible with future changes
    """
    convert._default_allow_truncated_result = enabled


def global_allow_split_transactions(enabled: bool):
    """
    call global_allow_truncated_result(False) for more safe execution and compatible with future changes
    """
    table._allow_split_transaction = enabled



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/ydb_scripting.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb._grpc.v4.protos import ydb_operation_pb2 as protos_dot_ydb__operation__pb2
from ydb._grpc.v4.protos import ydb_value_pb2 as protos_dot_ydb__value__pb2
from ydb._grpc.v4.protos import ydb_table_pb2 as protos_dot_ydb__table__pb2
from ydb._grpc.v4.protos import ydb_query_stats_pb2 as protos_dot_ydb__query__stats__pb2
from ydb._grpc.v4.protos import ydb_issue_message_pb2 as protos_dot_ydb__issue__message__pb2
from ydb._grpc.v4.protos import ydb_status_codes_pb2 as protos_dot_ydb__status__codes__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1aprotos/ydb_scripting.proto\x12\rYdb.Scripting\x1a\x1aprotos/ydb_operation.proto\x1a\x16protos/ydb_value.proto\x1a\x16protos/ydb_table.proto\x1a\x1cprotos/ydb_query_stats.proto\x1a\x1eprotos/ydb_issue_message.proto\x1a\x1dprotos/ydb_status_codes.proto\"\xa5\x02\n\x11\x45xecuteYqlRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0e\n\x06script\x18\x02 \x01(\t\x12\x44\n\nparameters\x18\x03 \x03(\x0b\x32\x30.Ydb.Scripting.ExecuteYqlRequest.ParametersEntry\x12;\n\rcollect_stats\x18\x04 \x01(\x0e\x32$.Ydb.Table.QueryStatsCollection.Mode\x1a\x42\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1e\n\x05value\x18\x02 \x01(\x0b\x32\x0f.Ydb.TypedValue:\x02\x38\x01\"B\n\x12\x45xecuteYqlResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"h\n\x10\x45xecuteYqlResult\x12#\n\x0bresult_sets\x18\x01 \x03(\x0b\x32\x0e.Ydb.ResultSet\x12/\n\x0bquery_stats\x18\x02 \x01(\x0b\x32\x1a.Ydb.TableStats.QueryStats\"\xa7\x01\n\x19\x45xecuteYqlPartialResponse\x12)\n\x06status\x18\x01 \x01(\x0e\x32\x19.Ydb.StatusIds.StatusCode\x12\'\n\x06issues\x18\x02 \x03(\x0b\x32\x17.Ydb.Issue.IssueMessage\x12\x36\n\x06result\x18\x03 \x01(\x0b\x32&.Ydb.Scripting.ExecuteYqlPartialResult\"\x88\x01\n\x17\x45xecuteYqlPartialResult\x12\x18\n\x10result_set_index\x18\x01 \x01(\r\x12\"\n\nresult_set\x18\x02 \x01(\x0b\x32\x0e.Ydb.ResultSet\x12/\n\x0bquery_stats\x18\x03 \x01(\x0b\x32\x1a.Ydb.TableStats.QueryStats\"\xc9\x01\n\x11\x45xplainYqlRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0e\n\x06script\x18\x02 \x01(\t\x12\x33\n\x04mode\x18\x03 \x01(\x0e\x32%.Ydb.Scripting.ExplainYqlRequest.Mode\"4\n\x04Mode\x12\x14\n\x10MODE_UNSPECIFIED\x10\x00\x12\x0c\n\x08VALIDATE\x10\x02\x12\x08\n\x04PLAN\x10\x03\"B\n\x12\x45xplainYqlResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\xb3\x01\n\x10\x45xplainYqlResult\x12N\n\x10parameters_types\x18\x01 \x03(\x0b\x32\x34.Ydb.Scripting.ExplainYqlResult.ParametersTypesEntry\x12\x0c\n\x04plan\x18\x02 \x01(\t\x1a\x41\n\x14ParametersTypesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x18\n\x05value\x18\x02 \x01(\x0b\x32\t.Ydb.Type:\x02\x38\x01\x42\x66\n\x12tech.ydb.scriptingB\x0fScriptingProtosZ<github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Scripting\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protos.ydb_scripting_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\022tech.ydb.scriptingB\017ScriptingProtosZ<github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Scripting\370\001\001'
  _EXECUTEYQLREQUEST_PARAMETERSENTRY._options = None
  _EXECUTEYQLREQUEST_PARAMETERSENTRY._serialized_options = b'8\001'
  _EXPLAINYQLRESULT_PARAMETERSTYPESENTRY._options = None
  _EXPLAINYQLRESULT_PARAMETERSTYPESENTRY._serialized_options = b'8\001'
  _EXECUTEYQLREQUEST._serialized_start=215
  _EXECUTEYQLREQUEST._serialized_end=508
  _EXECUTEYQLREQUEST_PARAMETERSENTRY._serialized_start=442
  _EXECUTEYQLREQUEST_PARAMETERSENTRY._serialized_end=508
  _EXECUTEYQLRESPONSE._serialized_start=510
  _EXECUTEYQLRESPONSE._serialized_end=576
  _EXECUTEYQLRESULT._serialized_start=578
  _EXECUTEYQLRESULT._serialized_end=682
  _EXECUTEYQLPARTIALRESPONSE._serialized_start=685
  _EXECUTEYQLPARTIALRESPONSE._serialized_end=852
  _EXECUTEYQLPARTIALRESULT._serialized_start=855
  _EXECUTEYQLPARTIALRESULT._serialized_end=991
  _EXPLAINYQLREQUEST._serialized_start=994
  _EXPLAINYQLREQUEST._serialized_end=1195
  _EXPLAINYQLREQUEST_MODE._serialized_start=1143
  _EXPLAINYQLREQUEST_MODE._serialized_end=1195
  _EXPLAINYQLRESPONSE._serialized_start=1197
  _EXPLAINYQLRESPONSE._serialized_end=1263
  _EXPLAINYQLRESULT._serialized_start=1266
  _EXPLAINYQLRESULT._serialized_end=1445
  _EXPLAINYQLRESULT_PARAMETERSTYPESENTRY._serialized_start=1380
  _EXPLAINYQLRESULT_PARAMETERSTYPESENTRY._serialized_end=1445
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/ydb_auth.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb._grpc.v3.protos import ydb_operation_pb2 as protos_dot_ydb__operation__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='protos/ydb_auth.proto',
  package='Ydb.Auth',
  syntax='proto3',
  serialized_options=b'\n\rtech.ydb.authZ7github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Auth\370\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x15protos/ydb_auth.proto\x12\x08Ydb.Auth\x1a\x1aprotos/ydb_operation.proto\"i\n\x0cLoginRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04user\x18\x02 \x01(\t\x12\x10\n\x08password\x18\x03 \x01(\t\"=\n\rLoginResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x1c\n\x0bLoginResult\x12\r\n\x05token\x18\x01 \x01(\tBK\n\rtech.ydb.authZ7github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Auth\xf8\x01\x01\x62\x06proto3'
  ,
  dependencies=[protos_dot_ydb__operation__pb2.DESCRIPTOR,])




_LOGINREQUEST = _descriptor.Descriptor(
  name='LoginRequest',
  full_name='Ydb.Auth.LoginRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation_params', full_name='Ydb.Auth.LoginRequest.operation_params', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user', full_name='Ydb.Auth.LoginRequest.user', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='password', full_name='Ydb.Auth.LoginRequest.password', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=63,
  serialized_end=168,
)


_LOGINRESPONSE = _descriptor.Descriptor(
  name='LoginResponse',
  full_name='Ydb.Auth.LoginResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation', full_name='Ydb.Auth.LoginResponse.operation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=170,
  serialized_end=231,
)


_LOGINRESULT = _descriptor.Descriptor(
  name='LoginResult',
  full_name='Ydb.Auth.LoginResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='token', full_name='Ydb.Auth.LoginResult.token', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=233,
  serialized_end=261,
)

_LOGINREQUEST.fields_by_name['operation_params'].message_type = protos_dot_ydb__operation__pb2._OPERATIONPARAMS
_LOGINRESPONSE.fields_by_name['operation'].message_type = protos_dot_ydb__operation__pb2._OPERATION
DESCRIPTOR.message_types_by_name['LoginRequest'] = _LOGINREQUEST
DESCRIPTOR.message_types_by_name['LoginResponse'] = _LOGINRESPONSE
DESCRIPTOR.message_types_by_name['LoginResult'] = _LOGINRESULT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LoginRequest = _reflection.GeneratedProtocolMessageType('LoginRequest', (_message.Message,), {
  'DESCRIPTOR' : _LOGINREQUEST,
  '__module__' : 'protos.ydb_auth_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Auth.LoginRequest)
  })
_sym_db.RegisterMessage(LoginRequest)

LoginResponse = _reflection.GeneratedProtocolMessageType('LoginResponse', (_message.Message,), {
  'DESCRIPTOR' : _LOGINRESPONSE,
  '__module__' : 'protos.ydb_auth_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Auth.LoginResponse)
  })
_sym_db.RegisterMessage(LoginResponse)

LoginResult = _reflection.GeneratedProtocolMessageType('LoginResult', (_message.Message,), {
  'DESCRIPTOR' : _LOGINRESULT,
  '__module__' : 'protos.ydb_auth_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Auth.LoginResult)
  })
_sym_db.RegisterMessage(LoginResult)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
